###########################################
############ FUNCAO tsanalysis ############
###########################################
#### Funo de anlise de sries temporais
#### Verso: 0.4
#### Data:   2010/10/04
#### Autor:  Renato Tadashi Izawa
###########################################

tsanalysis = function(modelo,alpha=.05,plot=TRUE,digits=4){
	if(!any(class(modelo)==c("Arima")))
		stop("O modelo no  da classe ARIMA.")
	  
	detG = function(values,typeG,n,interval){ # detalhes dos grficos
		corB = 4 # cor da banda de confiana
		traB = 3 # tipo de trao da banda de confiana
		corL = 2 # cor das linhas fora da banda de confiana
		groL = 2 # grossura das linhas fora da banda
		if (typeG == "p") groL = 1
		
		# detalhando abscissa
		if(n <= 24)			aux=seq(0,n,by=1)
		else if(n <= 36)	aux=seq(0,n,by=2)
		else if(n <= 72)	aux=seq(0,n,by=6)
		else 				aux=seq(0,n,by=12)
		axis(1,at=aux)
		abline(v=aux,lty=3,col=8)

		if(n <= 72)			axis(1,at=(1:n),lab=F,tcl=-0.2)
		else 				axis(1,at=seq(0,n,by=6),lab=F,tcl=-0.2)

		# banda de confiana
		abline(h=interval,lty=traB,col=corB)

		# pinta valores fora da banda de confiana
		aux = {}
		for(i in 1:length(values)){
			if(values[i]<interval[1] || values[i]>interval[2])
				aux = c(aux,values[i])
			else
				aux = c(aux,"")
		}

		lines(values,type=typeG,lwd=groL,pch="*")
		lines(aux,type=typeG,col=corL,lwd=groL,pch="*")
	}

	### Calculos das variveis
	scoreNormal = qnorm(alpha/2)
	model  = replace(modelo$residuals, which(is.na(modelo$residuals)), 0)
	nmodel = length(model)
	fac    = acf(model,plot=FALSE,na.action=na.pass)
	facp   = pacf(model,plot=FALSE,na.action=na.pass)

	nLag    = length(facp$acf)	# qtde de dados
	facLag  = fac$acf[-1]		# valores do FAC e retira o ndice 0, que sempre tem valor 1
	facpLag = facp$acf[1:nLag]	# valores do FACP

	facVar = 2 / fac$n.used^(.5)

	rm(fac,facp)

	ljungBox = {}
	for(i in 1:nLag) ljungBox = c(ljungBox,Box.test(model, lag = i, type="Ljung")$p.value)

	### Grficos
	if(plot){

		layout(rbind(c(1,1),c(2,3),c(4,5)))
		
		op2 = par(mar=c(3,5,1,1))

			# Srie
			interval = c(-1,1)*abs(scoreNormal)
			yLim = range(list(model/sd(model),interval), na.rm=T)

			aux = as.numeric(model/sd(model))
			plot(aux,ylim=yLim,xaxt="n",type="h",xlab="",ylab="Time x\nStandardized Residuals")
			detG(aux,"h",nmodel,interval)
			abline(h=0)

			# FAC e FACP
			interval = c(-1,1)*facVar
			yLim = range(list(facLag,facpLag,interval), na.rm=T)

			plot(facLag,ylim=yLim,xaxt="n",type="h",xlab="",ylab="Lags x ACF",main="")
			detG(facLag,"h",nLag,interval)
			abline(h=0)

			plot(facpLag,ylim=yLim,xaxt="n",type="h",xlab="",ylab="Lags x PACF",main="")
			detG(facpLag,"h",nLag,interval)
			abline(h=0)

			# Ljung-Box
			interval = c(alpha,2)
			yLim = c(0,1)
			
			plot(ljungBox,ylim=yLim,xaxt="n",xlab="",ylab="Lags x\nLjung-Box (p-value)",main="",pch="*")
			detG(ljungBox,"p",nLag,interval)

		par(op2)

		op2 = par(mar=c(3,1,1,1),cex=.8)
			plot(1:3,1:3,type="n",axes=FALSE)
			text(1,2.5,"Log Likelihood:",pos=4)
			text(3,2.5,round(modelo$loglik,digits),pos=2)
			text(1,2,"AIC:",pos=4)
			text(3,2,round(modelo$aic,digits),pos=2)
			text(1,1.5,"Sigma^2 Estimated:",pos=4)
			text(3,1.5,round(modelo$sigma2,digits),pos=2)

		par(op2)

	}
	
	res <- structure(.Data = list(call=match.call(),acf=facLag,pacf=facpLag,var.acf=facVar,ljung.box=ljungBox),class="tsanalysis")
	return(invisible(res))
}

print.tsanalysis = function(t){
	print(sort(names(t)))
}

